]> sigrok.org Git - libsigrok.git/blobdiff - strutil.c
sr: adjust copyright year
[libsigrok.git] / strutil.c
index e5278dfe3fda303156eb05b62bc95577f71b4158..5e8689e7a17768fc5db21ce5f80e6323edb3b97f 100644 (file)
--- a/strutil.c
+++ b/strutil.c
@@ -21,7 +21,8 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sigrok.h>
+#include "sigrok.h"
+#include "sigrok-internal.h"
 
 /**
  * Convert a numeric samplerate value to its "natural" string representation.
  * E.g. a value of 3000000 would be converted to "3 MHz", 20000 to "20 kHz".
  *
  * @param samplerate The samplerate in Hz.
* @return A malloc()ed string representation of the samplerate value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_samplerate_string(uint64_t samplerate)
+SR_API char *sr_samplerate_string(uint64_t samplerate)
 {
        char *o;
        int r;
 
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " GHz". */
-       if (!o)
+       /* Allocate enough for a uint64_t as string + " GHz". */
+       if (!(o = g_try_malloc0(30 + 1))) {
+               sr_err("strutil: %s: o malloc failed", __func__);
                return NULL;
+       }
 
        if (samplerate >= SR_GHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " GHz", samplerate / 1000000000);
@@ -52,7 +56,7 @@ char *sr_samplerate_string(uint64_t samplerate)
 
        if (r < 0) {
                /* Something went wrong... */
-               free(o);
+               g_free(o);
                return NULL;
        }
 
@@ -66,17 +70,20 @@ char *sr_samplerate_string(uint64_t samplerate)
  * E.g. a value of 3000000 would be converted to "3 us", 20000 to "50 ms".
  *
  * @param frequency The frequency in Hz.
* @return A malloc()ed string representation of the frequency value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_period_string(uint64_t frequency)
+SR_API char *sr_period_string(uint64_t frequency)
 {
        char *o;
        int r;
 
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " ms". */
-       if (!o)
+       /* Allocate enough for a uint64_t as string + " ms". */
+       if (!(o = g_try_malloc0(30 + 1))) {
+               sr_err("strutil: %s: o malloc failed", __func__);
                return NULL;
+       }
 
        if (frequency >= SR_GHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " ns", frequency / 1000000000);
@@ -89,7 +96,7 @@ char *sr_period_string(uint64_t frequency)
 
        if (r < 0) {
                /* Something went wrong... */
-               free(o);
+               g_free(o);
                return NULL;
        }
 
@@ -103,8 +110,8 @@ char *sr_period_string(uint64_t frequency)
  * @param triggerstring TODO
  * @return TODO
  */
-char **sr_parse_triggerstring(struct sr_device *device,
-                             const char *triggerstring)
+SR_API char **sr_parse_triggerstring(struct sr_device *device,
+                                    const char *triggerstring)
 {
        GSList *l;
        struct sr_probe *probe;
@@ -114,7 +121,12 @@ char **sr_parse_triggerstring(struct sr_device *device,
 
        max_probes = g_slist_length(device->probes);
        error = FALSE;
-       triggerlist = g_malloc0(max_probes * sizeof(char *));
+
+       if (!(triggerlist = g_try_malloc0(max_probes * sizeof(char *)))) {
+               sr_err("session file: %s: metafile malloc failed", __func__);
+               return NULL;
+       }
+
        tokens = g_strsplit(triggerstring, ",", max_probes);
        trigger_types = device->plugin->get_device_info(0, SR_DI_TRIGGER_TYPES);
        if (trigger_types == NULL)
@@ -138,7 +150,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
                }
 
                if (probenum < 1 || probenum > max_probes) {
-                       printf("Invalid probe.\n");
+                       sr_err("Invalid probe.\n");
                        error = TRUE;
                        break;
                }
@@ -146,7 +158,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
                if ((trigger = strchr(tokens[i], '='))) {
                        for (tc = ++trigger; *tc; tc++) {
                                if (strchr(trigger_types, *tc) == NULL) {
-                                       printf("Unsupported trigger type "
+                                       sr_err("Unsupported trigger type "
                                               "'%c'\n", *tc);
                                        error = TRUE;
                                        break;
@@ -160,8 +172,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
 
        if (error) {
                for (i = 0; i < max_probes; i++)
-                       if (triggerlist[i])
-                               g_free(triggerlist[i]);
+                       g_free(triggerlist[i]);
                g_free(triggerlist);
                triggerlist = NULL;
        }
@@ -180,19 +191,19 @@ char **sr_parse_triggerstring(struct sr_device *device,
  * Spaces (but not other whitespace) between value and suffix are allowed.
  *
  * @param sizestring A string containing a (decimal) size value.
- * @return The string's size value as uint64_t.
+ * @param size Pointer to uint64_t which will contain the string's size value.
+ * @return SR_OK or error code
  *
- * TODO: Error handling.
  */
-uint64_t sr_parse_sizestring(const char *sizestring)
+SR_API int sr_parse_sizestring(const char *sizestring, uint64_t *size)
 {
-       int multiplier;
-       uint64_t val;
+       int multiplier, done;
        char *s;
 
-       val = strtoull(sizestring, &s, 10);
+       *size = strtoull(sizestring, &s, 10);
        multiplier = 0;
-       while (s && *s && multiplier == 0) {
+       done = FALSE;
+       while (s && *s && multiplier == 0 && !done) {
                switch (*s) {
                case ' ':
                        break;
@@ -209,15 +220,18 @@ uint64_t sr_parse_sizestring(const char *sizestring)
                        multiplier = SR_GHZ(1);
                        break;
                default:
-                       val = 0;
-                       multiplier = -1;
+                       done = TRUE;
+                       s--;
                }
                s++;
        }
        if (multiplier > 0)
-               val *= multiplier;
+               *size *= multiplier;
 
-       return val;
+       if (*s && strcasecmp(s, "Hz"))
+               return SR_ERR;
+
+       return SR_OK;
 }
 
 /**
@@ -239,7 +253,7 @@ uint64_t sr_parse_sizestring(const char *sizestring)
  * TODO: picoseconds?
  * TODO: Allow both lower-case and upper-case.
  */
-uint64_t sr_parse_timestring(const char *timestring)
+SR_API uint64_t sr_parse_timestring(const char *timestring)
 {
        uint64_t time_msec;
        char *s;
@@ -261,3 +275,17 @@ uint64_t sr_parse_timestring(const char *timestring)
 
        return time_msec;
 }
+
+SR_API gboolean sr_parse_boolstring(const char *boolstr)
+{
+       if (!boolstr)
+               return FALSE;
+
+       if (!g_strcasecmp(boolstr, "true") ||
+           !g_strcasecmp(boolstr, "yes") ||
+           !g_strcasecmp(boolstr, "on") ||
+           !g_strcasecmp(boolstr, "1"))
+               return TRUE;
+
+       return FALSE;
+}
@@ -21,7 +21,8 @@
  *
  * @param samplerate The samplerate in Hz.
- * @return A malloc()ed string representation of the samplerate value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ * @return A g_try_malloc()ed string representation of the samplerate value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */
@@ -66,17 +70,20 @@ char *sr_samplerate_string(uint64_t samplerate)
  *
  * @param frequency The frequency in Hz.
- * @return A malloc()ed string representation of the frequency value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ * @return A g_try_malloc()ed string representation of the frequency value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */